import { __spread, __awaiter, __generator } from 'tslib';
import { InjectionToken, inject, PLATFORM_ID, isDevMode, Injectable, Inject, ɵɵdefineInjectable, ɵɵinject, APP_BOOTSTRAP_LISTENER, APP_INITIALIZER, NgModule } from '@angular/core';
import { Router, NavigationEnd } from '@angular/router';
import { isPlatformBrowser, DOCUMENT, CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * Generated from: lib/tokens/window-token.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Check if this environment can access Window object and return window or null if false.
 * @param {?} platformId
 * @return {?}
 */
function getWindow(platformId) {
    return (isPlatformBrowser(platformId))
        ? window : null;
}
/**
 * Provide DOM Window reference or null if the environment is not a Browser.
 * @type {?}
 */
var WINDOW = new InjectionToken('hj-window', {
    providedIn: 'root',
    factory: (/**
     * @return {?}
     */
    function () { return getWindow(inject(PLATFORM_ID)); }),
});

/**
 * @fileoverview added by tsickle
 * Generated from: lib/tokens/ngx-hj-token.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Check if there is some global function called gtag on Window object, or create an empty function to doesn't brake codes...
 * @param {?} window
 * @return {?}
 */
function getHjFn(window) {
    // // h.hj=h.hj||function(){(h.hj.q=h.hj.q||[]).push(arguments)};
    return (window)
        ? window.hj = (/** @type {?} */ ((window.hj || (/**
         * @return {?}
         */
        function () {
            (window.hj.q = window.hj.q || []).push(arguments);
        }))))
        : null;
}
/**
 * Provides an injection token to access Google Analytics Gtag Function
 * @type {?}
 */
var NGX_HJ_FN = new InjectionToken('ngx-hj-fn', {
    providedIn: 'root',
    factory: (/**
     * @return {?}
     */
    function () { return getHjFn(inject(WINDOW)); })
});

/**
 * @fileoverview added by tsickle
 * Generated from: lib/tokens/ngx-hotjar-settings.token.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Provides a token to override default settings. You can use this token to enhance our library
 * and configure multiple sites/app on the same environment.
 * @type {?}
 */
var NGX_HOTJAR_SETTINGS_TOKEN = new InjectionToken('ngx-hotjar-settings', {
    factory: (/**
     * @return {?}
     */
    function () { return ({ trackingCode: '', version: 6, ennableTracing: false }); })
});

/**
 * @fileoverview added by tsickle
 * Generated from: lib/services/ngx-hotjar.service.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var NgxHotjarService = /** @class */ (function () {
    /** @ignore */
    function NgxHotjarService(_hj, settings) {
        this._hj = _hj;
        this.settings = settings;
    }
    Object.defineProperty(NgxHotjarService.prototype, "lib", {
        /**
         * Provide direct access to the `hj.*` static functions. If the desired function is not available on type definition, you can cast to `any` as following.
         *
      ```typescript
      (hjService.lib as any).myBrandNewStaticFn()
      ```
         */
        get: /**
         * Provide direct access to the `hj.*` static functions. If the desired function is not available on type definition, you can cast to `any` as following.
         *
         * ```typescript
         * (hjService.lib as any).myBrandNewStaticFn()
         * ```
         * @return {?}
         */
        function () {
            return this._hj;
        },
        enumerable: true,
        configurable: true
    });
    /** Expose Hotjar Function calls */
    /**
     * Expose Hotjar Function calls
     * @param {...?} args
     * @return {?}
     */
    NgxHotjarService.prototype.hj = /**
     * Expose Hotjar Function calls
     * @param {...?} args
     * @return {?}
     */
    function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        try {
            this._hj.apply(this, __spread(args));
        }
        catch (err) {
            if (isDevMode() || this.settings.ennableTracing) {
                console.error(err.message);
            }
        }
    };
    /**
     * Fires an PageView event on Hotjar. Use this method to trigger an virtual url path. The same as
     *
  ```typescript
  hj('vpv', path)
  ```
     */
    /**
     * Fires an PageView event on Hotjar. Use this method to trigger an virtual url path. The same as
     *
     * ```typescript
     * hj('vpv', path)
     * ```
     * @param {?} path
     * @return {?}
     */
    NgxHotjarService.prototype.virtualPageView = /**
     * Fires an PageView event on Hotjar. Use this method to trigger an virtual url path. The same as
     *
     * ```typescript
     * hj('vpv', path)
     * ```
     * @param {?} path
     * @return {?}
     */
    function (path) {
        this.hj('vpv', path);
    };
    /**
     * Fires an event on Hotjar. Use this method to trigger events on forms and start video recordings. Same as
     *
  ```typescript
  hj('trigger', path)
  ```
     */
    /**
     * Fires an event on Hotjar. Use this method to trigger events on forms and start video recordings. Same as
     *
     * ```typescript
     * hj('trigger', path)
     * ```
     * @param {?} path
     * @return {?}
     */
    NgxHotjarService.prototype.trigger = /**
     * Fires an event on Hotjar. Use this method to trigger events on forms and start video recordings. Same as
     *
     * ```typescript
     * hj('trigger', path)
     * ```
     * @param {?} path
     * @return {?}
     */
    function (path) {
        this.hj('trigger', path);
    };
    /**
     * @param {?} tagOrCollection
     * @param {...?} tags
     * @return {?}
     */
    NgxHotjarService.prototype.tagRecording = /**
     * @param {?} tagOrCollection
     * @param {...?} tags
     * @return {?}
     */
    function (tagOrCollection) {
        var tags = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            tags[_i - 1] = arguments[_i];
        }
        // Retrocompatibility
        if (!Array.isArray(tagOrCollection)) {
            tagOrCollection = [tagOrCollection];
        }
        this.hj('tagRecording', tagOrCollection.concat.apply(tagOrCollection, __spread(tags)));
    };
    /**
     * This option is available in case you need to set up page change tracking manually
     * within your app's router.
     *
  ```typescript
  hj('stateChange', path)
  ```
     */
    /**
     * This option is available in case you need to set up page change tracking manually
     * within your app's router.
     *
     * ```typescript
     * hj('stateChange', path)
     * ```
     * @param {?} path
     * @return {?}
     */
    NgxHotjarService.prototype.stateChange = /**
     * This option is available in case you need to set up page change tracking manually
     * within your app's router.
     *
     * ```typescript
     * hj('stateChange', path)
     * ```
     * @param {?} path
     * @return {?}
     */
    function (path) {
        this.hj('stateChange', path);
    };
    /**
     * Signals form submission success
     *
  ```typescript
  hj('formSubmitSuccessful');
  ```
     */
    /**
     * Signals form submission success
     *
     * ```typescript
     * hj('formSubmitSuccessful');
     * ```
     * @return {?}
     */
    NgxHotjarService.prototype.formSubmitSuccessful = /**
     * Signals form submission success
     *
     * ```typescript
     * hj('formSubmitSuccessful');
     * ```
     * @return {?}
     */
    function () {
        this.hj('formSubmitSuccessful');
    };
    /**
     * Signals form submission failure
     *
  ```typescript
  hj('formSubmitFailed');
  ```
     */
    /**
     * Signals form submission failure
     *
     * ```typescript
     * hj('formSubmitFailed');
     * ```
     * @return {?}
     */
    NgxHotjarService.prototype.formSubmitFailed = /**
     * Signals form submission failure
     *
     * ```typescript
     * hj('formSubmitFailed');
     * ```
     * @return {?}
     */
    function () {
        this.hj('formSubmitFailed');
    };
    NgxHotjarService.decorators = [
        { type: Injectable, args: [{
                    providedIn: 'root'
                },] }
    ];
    /** @nocollapse */
    NgxHotjarService.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Inject, args: [NGX_HJ_FN,] }] },
        { type: undefined, decorators: [{ type: Inject, args: [NGX_HOTJAR_SETTINGS_TOKEN,] }] }
    ]; };
    /** @nocollapse */ NgxHotjarService.ngInjectableDef = ɵɵdefineInjectable({ factory: function NgxHotjarService_Factory() { return new NgxHotjarService(ɵɵinject(NGX_HJ_FN), ɵɵinject(NGX_HOTJAR_SETTINGS_TOKEN)); }, token: NgxHotjarService, providedIn: "root" });
    return NgxHotjarService;
}());
if (false) {
    /**
     * @ignore
     * @type {?}
     * @private
     */
    NgxHotjarService.prototype._hj;
    /**
     * @ignore
     * @type {?}
     * @private
     */
    NgxHotjarService.prototype.settings;
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/initializers/hotjar-router.initializer.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Provide a DI Configuration to attach Hotjar Trigger to Router Events at Angular Startup Cycle.
 * @type {?}
 */
var NGX_HOTJAR_ROUTER_INITIALIZER_PROVIDER = {
    provide: APP_BOOTSTRAP_LISTENER,
    multi: true,
    useFactory: HotjarRouterInitializer,
    deps: [
        NgxHotjarService
    ]
};
/**
 * Attach a listener to `NavigationEnd` Router event. So, every time Router finish the page resolution it should call `NavigationEnd` event.
 * We assume that NavigationEnd is the final page resolution and call Hotjar `stateChange` command.
 *
 * To avoid double binds, we also destroy the subscription when de Bootstrap Component is destroied. But, we don't know for sure
 * that this strategy does not cause double bind on multiple bootstrap components.
 *
 * We are using de component's injector reference to resolve Router, sou I hope there is no problem w/ double bing.
 *
 * If you have this problem, I encourage not Use NgxHotjarRouterModule and atach the listener on AppComponent initialization.
 * @param {?} hjService
 * @return {?}
 * @this {*}
 */
function HotjarRouterInitializer(hjService) {
    var _this = this;
    return (/**
     * @param {?} c
     * @return {?}
     */
    function (c) { return __awaiter(_this, void 0, void 0, function () {
        var router, subs;
        return __generator(this, function (_a) {
            router = c.injector.get(Router);
            subs = router
                .events
                .subscribe((/**
             * @param {?} event
             * @return {?}
             */
            function (event) {
                if (event instanceof NavigationEnd) {
                    hjService.stateChange(event.urlAfterRedirects);
                }
            }));
            // Cleanup
            c.onDestroy((/**
             * @return {?}
             */
            function () { return subs.unsubscribe(); }));
            return [2 /*return*/];
        });
    }); });
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/initializers/hotjar.initializer.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Provides a TOKEN to manually configure Hojtar tracking code by angular way.
 * @type {?}
 */
var NGX_HOTJAR_INITIALIZER_PROVIDER = {
    provide: APP_INITIALIZER,
    multi: true,
    useFactory: HotjarInitializer,
    deps: [
        NGX_HOTJAR_SETTINGS_TOKEN,
        DOCUMENT,
        WINDOW
    ]
};
/**
 * Configuration Factory to create hotjar install script tag and attache on DOM at angular initialization.
 * @param {?} settings
 * @param {?} document
 * @param {?} window
 * @return {?}
 * @this {*}
 */
function HotjarInitializer(settings, document, window) {
    var _this = this;
    return (/**
     * @return {?}
     */
    function () { return __awaiter(_this, void 0, void 0, function () {
        var hjWindow, head, script, uri;
        return __generator(this, function (_a) {
            if (!settings.trackingCode) {
                if (isDevMode()) {
                    console.error('Empty tracking code for Hotjar. Make sure to provide one when initializing NgxHotjarModule.');
                }
                return [2 /*return*/];
            }
            if (!document) {
                if (isDevMode()) {
                    console.error('Was not possible to access `document` instance. Make shure this environment works on a Broser like API');
                }
                return [2 /*return*/];
            }
            if (!window) {
                if (isDevMode()) {
                    console.error('Was not possible to access `window` api. Make sure this environment works like a browser.');
                }
                return [2 /*return*/];
            }
            // (function(h: any, o: any, t: any, j: any, a?: any, r?: any) {
            // h._hjSettings = { hjid: $settings.trackingCode, hjsv: $settings.version || 6 };
            hjWindow = Object.defineProperty(window, '_hjSettings', { value: { hjid: settings.trackingCode, hjsv: settings.version || 6 } });
            head = document.querySelector('head');
            script = document.createElement('script');
            uri = "https://static.hotjar.com/c/hotjar-" + hjWindow._hjSettings.hjid + ".js?sv=" + hjWindow._hjSettings.hjsv;
            script.async = true;
            script.src = (settings.uri || uri);
            head.appendChild(script);
            return [2 /*return*/];
        });
    }); });
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/interfaces/i-hotjar-settings.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Ngx Htjar Confguration Parameters.
 * @record
 */
function IHotjarSettings() { }
if (false) {
    /**
     * Hotjar tracking code, aka, SITE ID
     * @type {?}
     */
    IHotjarSettings.prototype.trackingCode;
    /**
     * Script Versrion, default 6
     * @type {?|undefined}
     */
    IHotjarSettings.prototype.version;
    /**
     * When ennable, trace all miss `hj()` call.
     * @type {?|undefined}
     */
    IHotjarSettings.prototype.ennableTracing;
    /**
     * Override default URI
     * @type {?|undefined}
     */
    IHotjarSettings.prototype.uri;
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/types/hj.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Hotjar Library
 * @record
 */
function HjFn() { }
if (false) {
    /** @type {?} */
    HjFn.prototype.q;
    /* Skipping unhandled member: (...args: Array<any>): void;*/
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/types/hj-bind.type.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * Generated from: lib/ngx-hotjar.module.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Install Hotjar scripts on Angular Startup life cycle if this environment is a Browser, otherwise just ignore this step.
 *
 * You shall add this module on the Gighest level module of your application, aka `AppModule`. When
 * setup this module, you also have to provide you hotjat tracking code and the version of script. The default version
 * is 6. We do not recoment to expose the tracking code in the repository, so please, use angular environment variable.
 *
 * ## Exemple of Use
 *
 * ```typescript
 * \\@NgModule({
 * ...
 * imports: [
 * ...
 * NgxHotjarModule.forRoot(envorinment.hj)
 * // ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 * ]
 * })
 * ```
 */
var NgxHotjarModule = /** @class */ (function () {
    function NgxHotjarModule() {
    }
    /**
     * Setup global settings and provide Hotjar Services.
     *
     * You private tracking code. This tracking code is also known as the same number as `SITE ID` inside Hotjar Dashboard.
     */
    /**
     * Setup global settings and provide Hotjar Services.
     *
     * You private tracking code. This tracking code is also known as the same number as `SITE ID` inside Hotjar Dashboard.
     * @param {?} trackingCode
     * @param {?=} version
     * @param {?=} uri
     * @return {?}
     */
    NgxHotjarModule.forRoot = /**
     * Setup global settings and provide Hotjar Services.
     *
     * You private tracking code. This tracking code is also known as the same number as `SITE ID` inside Hotjar Dashboard.
     * @param {?} trackingCode
     * @param {?=} version
     * @param {?=} uri
     * @return {?}
     */
    function (trackingCode, version, uri) {
        if (version === void 0) { version = 6; }
        return {
            ngModule: NgxHotjarModule,
            providers: [
                {
                    provide: NGX_HOTJAR_SETTINGS_TOKEN,
                    useValue: {
                        trackingCode: trackingCode,
                        version: version,
                        uri: uri
                    }
                },
                NGX_HOTJAR_INITIALIZER_PROVIDER,
            ]
        };
    };
    NgxHotjarModule.decorators = [
        { type: NgModule, args: [{
                    imports: [],
                    declarations: [],
                    exports: [],
                    providers: []
                },] }
    ];
    return NgxHotjarModule;
}());

/**
 * @fileoverview added by tsickle
 * Generated from: lib/ngx-hotjar-router/ngx-hotjar-router.module.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Attach a listener to `NavigationEnd` Router event. So, every time Router finish the page resolution it should call `NavigationEnd` event.
 * We assume that NavigationEnd is the final page resolution and call Hotjar `stateChange` command.
 *
 * To avoid double binds, we also destroy the subscription when de Bootstrap Component is destroied. But, we don't know for sure
 * that this strategy does not cause double bind on multiple bootstrap components.
 *
 * We are using de component's injector reference to resolve Router, sou I hope there is no problem w/ double bing.
 *
 * If you have this problem, I encourage not Use NgxHotjarRouterModule and atach the listener on AppComponent initialization.
 *
 * This Module is just a sugar for:
 *
 * ```typescript
 * constructor(private router: Router) {}
 * ...
 * ngOnInit() {
 * ...
 * this.router
 * .events
 * .pipe(takeUntil(this.onDestroy$))
 * .subscribe(event => {
 * if (event instanceof NavigationEnd) {
 * hjService.pageView(event.urlAfterRedirects, undefined);
 * }
 * });
 * ```
 */
var NgxHotjarRouterModule = /** @class */ (function () {
    function NgxHotjarRouterModule() {
    }
    NgxHotjarRouterModule.decorators = [
        { type: NgModule, args: [{
                    imports: [
                        CommonModule,
                        NgxHotjarModule
                    ],
                    providers: [
                        NGX_HOTJAR_ROUTER_INITIALIZER_PROVIDER
                    ],
                    declarations: []
                },] }
    ];
    return NgxHotjarRouterModule;
}());

/**
 * @fileoverview added by tsickle
 * Generated from: public_api.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * Generated from: ngx-hotjar.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { HotjarInitializer, HotjarRouterInitializer, NGX_HJ_FN, NGX_HOTJAR_INITIALIZER_PROVIDER, NGX_HOTJAR_ROUTER_INITIALIZER_PROVIDER, NGX_HOTJAR_SETTINGS_TOKEN, NgxHotjarModule, NgxHotjarRouterModule, NgxHotjarService, WINDOW, getHjFn, getWindow };
//# sourceMappingURL=ngx-hotjar.js.map
